#!/usr/bin/perl -w

# Copyright 2017, 2018, 2020 Kevin Ryde

# This file is part of nautyextra.
#
# nautyextra is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3, or (at your option) any later
# version.
#
# nautyextra is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with nautyextra.  If not, see <http://www.gnu.org/licenses/>.


use strict;
use FindBin;
use File::Slurp;
use Tie::IxHash;
use Regexp::Common 'comment';

# uncomment this to run the ### lines
# use Smart::Comments;

sub pos_linenum {
  my ($str, $pos) = @_;
  $pos //= pos($_[0]);
  $str = substr($str, 0, $pos);
  return 1 + scalar($str =~ tr/\n//);
}

# return $str with all characters except \n newlines deleted
sub str_leave_newlines {
  my ($str) = @_;
  $str =~ tr/\n//cd;
  return $str;
}

my $C_comment_re = $RE{'comment'}{'C'};

my $bad = 0;

foreach my $filename ('nautyextra.c', 'nautyextra-wip.c') {
  my $full_filename = File::Spec->catfile($FindBin::Bin, File::Spec->updir,
                                          $filename);
  my $rel_filename = File::Spec->abs2rel($full_filename);
  my $str = File::Slurp::read_file($full_filename);
  $str =~ s{$C_comment_re}{str_leave_newlines($&)}ges;  # no comments
  my $count_funcs = 0;
  my $count_dyns = 0;
  my $func = 'no function';
  while ($str =~ /(?<Func>^(nautyextra|internal)_\w+)
                |(DYNALLOC|WORKSPACE)[^)]*?\"(?<Dyn>[^\"]*)\"/xmg) {
    if (defined $+{'Func'}) {
      $func = $+{'Func'};
      $func =~ s/^internal/nautyextra/;
      $count_funcs++;
    } else {
      my $dyn = $+{'Dyn'} // die;
      $count_dyns++;
      if ($dyn ne $func) {
        my $linenum = pos_linenum($str);
        print "$rel_filename:$linenum: DYNALLOC name \"$dyn\" in function $func\n";
        $bad++;
      }
    }
  }
  print "count $filename funcs $count_funcs dyns $count_dyns\n";
}

exit ($bad ? 1 : 0);
